# -*- coding: utf-8 -*-
# Copyright 2010-2013 Kolab Systems AG (http://www.kolabsys.com)
#
# Jeroen van Meeuwen (Kolab Systems) <vanmeeuwen a kolabsys.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 or, at your option, any later version
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

import sys

import commands

import pykolab

from pykolab.imap import IMAP
from pykolab.translate import _

log = pykolab.getLogger('pykolab.cli')
conf = pykolab.getConf()

def __init__():
    commands.register('summarize_quota_allocation', execute, description=description(), aliases=['sqa'])

def cli_options():
    my_option_group = conf.add_cli_parser_option_group(_("CLI Options"))
    my_option_group.add_option( '--server',
                                dest    = "connect_server",
                                action  = "store",
                                default = None,
                                metavar = "SERVER",
                                help    = _("List mailboxes on server SERVER only."))

def description():
    return """Summarize all quota allocated."""

def execute(*args, **kw):
    """
        Summarize all quota allocated.
    """

    imap = IMAP()

    if not conf.connect_server == None:
        imap.connect(server=conf.connect_server)
    else:
        imap.connect()

    quota_folders = imap.list_folders('*')

    quota_alloc = 0
    quota_used = 0

    for quota_folder in quota_folders:
        try:
            (used, quota) = imap.get_quota(quota_folder)
            if not used == None and not quota == None:
                quota_alloc += float(quota)
                used += float(used)
        except:
            pass

    print "Quota Allocated:\t%-5d GB" % ((quota_alloc / 1024 / 1024))
    print "Quota Used:\t%-5d GB" % ((quota_alloc / 1024 / 1024))
