# -*- coding: utf-8 -*-
# Copyright 2010-2013 Kolab Systems AG (http://www.kolabsys.com)
#
# Jeroen van Meeuwen (Kolab Systems) <vanmeeuwen a kolabsys.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 3 or, at your option, any later version
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#

import sys

import commands

import pykolab

from pykolab.auth import Auth
from pykolab.imap import IMAP
from pykolab.translate import _

log = pykolab.getLogger('pykolab.cli')
conf = pykolab.getConf()

def __init__():
    commands.register('rebalance_mailboxes', execute, description=description())

def cli_options():
    my_option_group = conf.add_cli_parser_option_group(_("CLI Options"))
    my_option_group.add_option( '--dry-run',
                                dest    = "dry_run",
                                action  = "store_true",
                                default = False,
                                help    = _("Do a dry run."))

def description():
    return """List quota for a folder."""

def execute(*args, **kw):
    """
        List quota for a mailbox
    """

    imap = IMAP()
    imap.connect()

    import os
    if not os.access('./cache.folders', os.R_OK):
        auth = Auth()
        auth.connect()

        domains = auth.list_domains()

        folders = {}

        for primary,secondaries in domains:
            for folder in imap.lm("user/%%@%s" % (primary)):
                folders[folder] = get_quota(imap, folder)

            for folder in imap.lm("shared/%%@%s" % (primary)):
                folders[folder] = get_quota(imap, folder)

            for secondary in secondaries:
                for folder in imap.lm("user/%%@%s" % (secondary)):
                    folders[folder] = get_quota(imap, folder)

                for folder in imap.lm("shared/%%@%s" % (secondary)):
                    folders[folder] = get_quota(imap, folder)

        for folder in imap.lm("user/%%"):
            folders[folder] = get_quota(imap, folder)

        for folder in imap.lm("shared/%%"):
            folders[folder] = get_quota(imap, folder)

        for folder in folders.keys():
            if folders[folder] == None:
                del folders[folder]

        for folder in folders:
            folders[folder]['server'] = imap.imap.find_mailfolder_server(folder)

        import json
        fp = open('./cache.folders', 'w')
        fp.write(json.dumps(folders))
        fp.close()

    else:
        import json
        fp = open('./cache.folders', 'r')
        folders = json.load(fp)
        fp.close()

    # Data gathered, calculations can begin.
    mboxs = {}
    used = {}
    alloc = {}

    for folder in folders:
        if not mboxs.has_key(folders[folder]['server']):
            mboxs[folders[folder]['server']] = 0

        if not used.has_key(folders[folder]['server']):
            used[folders[folder]['server']] = 0

        if not alloc.has_key(folders[folder]['server']):
            alloc[folders[folder]['server']] = 0

        mboxs[folders[folder]['server']] += 1
        used[folders[folder]['server']] += folders[folder]['used']
        alloc[folders[folder]['server']] += folders[folder]['quota']

    #mboxs = {'bs101.kolabsys.net': 2812, 'bs102.kolabsys.net': 46, 'bs103.kolabsys.net': 6}
    #used = {'bs101.kolabsys.net': 7508435, 'bs102.kolabsys.net': 586967, 'bs103.kolabsys.net': 121}
    #alloc = {'bs101.kolabsys.net': 3279032832, 'bs102.kolabsys.net': 60817408, 'bs103.kolabsys.net': 6291456}

    print mboxs
    print sum(mboxs.values()) / len(mboxs.keys())
    for (k,v) in mboxs.iteritems():
        print "k:", k, "is", float(v * len(mboxs.keys())) / float(sum(mboxs.values())), "%"

    print used
    print sum(used.values()) / len(used.keys())
    for (k,v) in used.iteritems():
        print "k:", k, "is", float(v * len(used.keys())) / float(sum(used.values())), "%"
        print "average storage per mailbox:", float(used[k]) / float(mboxs[k])

    print alloc
    print sum(alloc.values()) / len(alloc.keys())
    for (k,v) in alloc.iteritems():
        print "k:", k, "is", float(v * len(alloc.keys())) / float(sum(alloc.values())), "%"
        print "average quota per mailbox:", float(alloc[k]) / float(mboxs[k])


def get_quota(imap, folder):
    try:
        (used, quota) = imap.get_quota(folder)
        if not used == None and not quota == None:
            return {'used': used, 'quota': quota}
    except:
        return None
